package cn.com.qmth.examcloud.bridge.modules.cloudmarking.controller;

import cn.com.qmth.examcloud.bridge.modules.cloudmarking.bean.Block;
import cn.com.qmth.examcloud.bridge.modules.cloudmarking.bean.Section;
import cn.com.qmth.examcloud.bridge.modules.cloudmarking.bean.Sections;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.IOUtil;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.commons.util.ZipUtil;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamRecordCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamRecordQuestionsCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.SubjectiveAnswerBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.GetSubjectiveAnswerReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.GetToBeMarkExamRecordReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.GetSubjectiveAnswerResp;
import cn.com.qmth.examcloud.core.oe.admin.api.response.GetToBeMarkExamRecordResp;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import com.google.common.collect.Maps;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.File;
import java.nio.charset.Charset;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @Description 云阅卷服务端接口
 * 错误码范围(101001 至 101999)
 * @Author lideyin
 * @Date 2019/11/20 14:49
 * @Version 1.0
 */
@RestController
@RequestMapping("cmc")
public class CloudMarkingController {
    @Autowired
    ExamRecordQuestionsCloudService examRecordQuestionsCloudService;

    @Autowired
    SystemProperties systemConfig;

    @Autowired
    ExamRecordCloudService examRecordCloudService;

    /**
     * 根据考生获取待阅卷的主观题作答记录
     *
     * @param examStudentId 考生id
     * @return
     */
    @GetMapping(value = "getSubjectiveAnswer/{examStudentId}")
    public GetToBeMarkExamRecordResp getSubjectiveAnswer(@PathVariable Long examStudentId) {
        if (null == examStudentId) {
            throw new StatusException("101001", "考生id不允许为空");
        }

        GetToBeMarkExamRecordReq req = new GetToBeMarkExamRecordReq();
        req.setExamStudentIdList(Arrays.asList(examStudentId));

        return examRecordCloudService.getToBeMarkExamRecord(req);
    }

    /**
     * 批量生成主观题答案，以压缩文件方式返回
     */
    @GetMapping(value = "batchGetAnswer/{examId}")
    public ResponseEntity<?> batchGetSubjectiveAnswer(@PathVariable Long examId) {
        if (null == examId) {
            throw new StatusException("101001", "考试批次id不允许为空");
        }

        //答案临时存放目录
        final String tempAnswerDir = systemConfig.getTempDataDir() + "/tempAnswer" + System.currentTimeMillis();

        Long startId = 0L;
        Boolean hasData = false;
        while (true) {
            GetSubjectiveAnswerReq req = new GetSubjectiveAnswerReq();
            req.setExamId(examId);
            req.setStartExamRecordId(startId);

            GetSubjectiveAnswerResp resp = examRecordQuestionsCloudService.getSubjectiveAnswerList(req);
//            GetSubjectiveAnswerResp resp = JsonUtil.fromJson(testJson(), GetSubjectiveAnswerResp.class);
            if (resp.getNextExamRecordDataId().equals(startId)) {
                break;
            }

            if (!hasData) {
                hasData = true;
            }

            List<SubjectiveAnswerBean> subjectiveAnswerList = resp.getSubjectiveAnswerList();

            List<Long> examStudentIdList = subjectiveAnswerList.stream().
                    map(p -> p.getExamStudentId()).
                    distinct().collect(Collectors.toList());

            String courseCode = subjectiveAnswerList.get(0).getCourseCode();

            for (int i = 0; i < examStudentIdList.size(); i++) {
                Long examStudentId = examStudentIdList.get(i);

                List<Map<String, Object>> resultMapList = new ArrayList<>();

                //当前学生的作答记录
                List<SubjectiveAnswerBean> curStuAnswerList = subjectiveAnswerList.
                        stream().
                        filter(p -> p.getExamStudentId().equals(examStudentId)).
                        collect(Collectors.toList());

                for (SubjectiveAnswerBean sab : curStuAnswerList) {
                    Map<String, Object> resultMap = Maps.newHashMap();
                    //构建基本信息
                    resultMap.put("mainNumber", sab.getMainNumber());//大题号
                    resultMap.put("subNumber", sab.getOrder());//小题号

                    //构建题干
                    resultMap.put("body",getBodyOrAnswer(sab.getBody()));

                    //构建标准答案
                    resultMap.put("answer",getBodyOrAnswer(sab.getAnswer()));

                    //构建学生作答
                    resultMap.put("studentAnswer", buildStudentAnswerMap(sab));

                    resultMapList.add(resultMap);
                }

                //将考生结果保存至临时文件
                String json = JsonUtil.toJson(resultMapList);
                String filePath = tempAnswerDir + "/" + examId + "-" + courseCode + "-" + examStudentId + ".json";
                IOUtil.toFile(json.getBytes(Charset.forName("UTF-8")), filePath);
            }
            startId = resp.getNextExamRecordDataId();
        }

        if (!hasData) {
            return ResponseEntity.ok().body("There's no data to download.");
        }

        String zipPath = tempAnswerDir + "/" + examId + "-" + System.currentTimeMillis() + ".zip";
        File zipFile = new File(zipPath);
        ZipUtil.zip(new File(tempAnswerDir), zipFile);

        byte[] bytes = IOUtil.toByteArray(zipFile);

        String downLoadFileName = "exam-" + examId + ".zip";
        ResponseEntity<byte[]> responseEntity = ResponseEntity.ok().
                header("Content-Disposition", "attachment; filename=" + downLoadFileName).
                contentType(MediaType.APPLICATION_OCTET_STREAM).body(bytes);

        clearTempAnswerFile(tempAnswerDir);
        return responseEntity;
    }

    private Map<String, Object> buildStudentAnswerMap(SubjectiveAnswerBean sab) {
        //构建学生作答
        Map<String, Object> studentAnswerMap = Maps.newHashMap();

        List<Map<String, Object>> sectionMapList = new ArrayList<>();
        Map<String, Object> sectionMap = Maps.newHashMap();
        List<Map<String, Object>> blockMapList = new ArrayList<>();

        Map<String, Object> blockMap = Maps.newHashMap();
        blockMap.put("type", sab.getRealAnswerType());
        blockMap.put("value", sab.getStudentAnswer());
        blockMapList.add(blockMap);

        sectionMap.put("blocks", blockMapList);
        sectionMapList.add(sectionMap);

        studentAnswerMap.put("sections", sectionMapList);
        return studentAnswerMap;
    }

    /**
     * 获取格式化后后的题干或标准答案
     * @param str
     * @return
     */
    private Sections getBodyOrAnswer(String str) {
        Sections body = new Sections();
        List<Section> sections = new ArrayList<>();
        // 得到小题题干或者答案行数
        if (org.apache.commons.lang3.StringUtils.isBlank(str)) {
            return body;
        }
        String[] questionRowStrings = str.split("</p>");
        for (int i = 0; i < questionRowStrings.length; i++) {
            List<Block> blocks = disposeQuestionBodyOrOption(questionRowStrings[i]);
            if (blocks != null && blocks.size() > 0) {
                Section section = new Section();
                // 将小题题干拆分为Block集合
                section.setBlocks(blocks);
                sections.add(section);
            }
        }
        body.setSections(sections);
        return body;
    }

    private List<Block> disposeQuestionBodyOrOption(String questionRow) {
        List<Block> blocks = new ArrayList<>();
        // 去掉每行里面的<p>,<span>,</span>标签
        questionRow = questionRow.replaceAll("<p>", "").replaceAll("</p>", "").replaceAll("<span>", "")
                .replaceAll("</span>", "").replaceAll("</a>", "");
        String[] questionRowStrings = questionRow.split("<|/>|>");
        for (int i = 0; i < questionRowStrings.length; i++) {
            Block block = new Block();
            String rowStr = questionRowStrings[i];
            // 判断是否有图片
            if (rowStr.startsWith("img")) {
                rowStr = "<" + rowStr + ">";
                Map<String, Object> param = new HashMap<>();
                // 需要继续做截取，取到Parma
                block.setType("image");
                // 获取图片的路径
                List<String> strSrcList = getImg(rowStr, "src");
                if (strSrcList.size() > 0) {
                    String strSrc = strSrcList.get(0).replaceAll("src=\"", "").replaceAll("\"", "");
                    block.setValue(strSrc);
                }
                // 获取图片的高度
                List<String> strHeightList = getImg(rowStr, "height");
                if (strHeightList.size() > 0) {
                    String strHeight = strHeightList.get(0).replaceAll("height=\"", "").replaceAll("\"", "");
                    param.put("height", strHeight);
                }
                // 获取图片的宽度
                List<String> strWidthList = getImg(rowStr, "width");
                if (strHeightList.size() > 0) {
                    String strWidth = strWidthList.get(0).replaceAll("width=\"", "").replaceAll("\"", "");
                    param.put("width", strWidth);
                }
                block.setParam(param);
                blocks.add(block);
            } else if (rowStr.startsWith("a") && rowStr.contains("id") && rowStr.contains("name")) { // 处理音频
                rowStr = "<" + rowStr + ">";
                block.setPlayTime(1);
                block.setType("audio");
                block.setValue(this.getAttrValue(rowStr, "id"));
                blocks.add(block);
            } else {
                block.setType("text");
                rowStr = rowStr.replace("&nbsp;", "");// 消除空格
                rowStr = rowStr.replace("&quot;", "\"");// 将&quot;转换成\"
                rowStr = rowStr.replace("&lt;", "<");// 将&lt;转换成<
                rowStr = rowStr.replace("&gt;", ">");// 将&gt;转换成>
                rowStr = rowStr.replace("&amp;", "&");// 将&amp;转换成&
                if (org.apache.commons.lang3.StringUtils.isNotBlank(rowStr)) {
                    block.setValue(rowStr);
                    blocks.add(block);
                }
            }
        }

        return blocks;
    }

    /**
     * 获取图片里面的路径，长度，宽度
     */
    private List<String> getImg(String s, String str) {
        String regex;
        List<String> list = new ArrayList<>();
        regex = str + "=\"(.*?)\"";
        Pattern pa = Pattern.compile(regex, Pattern.DOTALL);
        Matcher ma = pa.matcher(s);
        while (ma.find()) {
            list.add(ma.group());
        }
        return list;
    }

    private String getAttrValue(String questionStr, String attrName) {
        Pattern aPattern = Pattern.compile("a.*");
        Matcher aMatcher = aPattern.matcher(questionStr);

        if (aMatcher.find()) {
            String idRegex = attrName + "=\".*?\"";
            Pattern idPattern = Pattern.compile(idRegex);
            Matcher idMatcher = idPattern.matcher(aMatcher.group());
            if (idMatcher.find()) {
                return idMatcher.group()
                        .replaceAll(attrName + "=\"", "")
                        .replaceAll("\"", "");
            }
        }

        return "";
    }

    /**
     * 清空临时目录中的文件
     *
     * @param tempAnswerDir
     */
    private void clearTempAnswerFile(String tempAnswerDir) {
        File t1 = new File(tempAnswerDir);
        for (File f : t1.listFiles()) {
            FileUtils.deleteQuietly(f);
        }
    }

    private String testJson() {
        return "{\n" +
                "    \"cost\": 6317,\n" +
                "    \"nextExamRecordDataId\": 80703,\n" +
                "    \"subjectiveAnswerList\": [{\n" +
                "        \"examId\": 198,\n" +
                "        \"courseCode\": \"6000459\",\n" +
                "        \"examStudentId\": 1837550,\n" +
                "        \"examRecordDataId\": 80507,\n" +
                "        \"mainNumber\": 5,\n" +
                "        \"questionId\": \"5c45384def8fce45257834d8\",\n" +
                "        \"order\": 19,\n" +
                "        \"studentAnswer\": \"https://ecs-test-static.qmth.com.cn/oe-answer-file/1837550/80507/19/1837550_80507_19_1558433562519.jpg|https://ecs-test-static.qmth.com.cn/oe-answer-file/1837550/80507/19/1837550_80507_19_1558433889599.png\",\n" +
                "        \"answerType\": \"SINGLE_AUDIO\",\n" +
                "        \"realAnswerType\": \"audio\"\n" +
                "    }, {\n" +
                "        \"examId\": 198,\n" +
                "        \"courseCode\": \"6000459\",\n" +
                "        \"examStudentId\": 1837550,\n" +
                "        \"examRecordDataId\": 80507,\n" +
                "        \"mainNumber\": 5,\n" +
                "        \"questionId\": \"5c45384def8fce45257834d9\",\n" +
                "        \"order\": 20,\n" +
                "        \"studentAnswer\": null,\n" +
                "        \"answerType\": \"SINGLE_AUDIO\",\n" +
                "        \"realAnswerType\": \"audio\"\n" +
                "    }, {\n" +
                "        \"examId\": 198,\n" +
                "        \"courseCode\": \"6000459\",\n" +
                "        \"examStudentId\": 1837550,\n" +
                "        \"examRecordDataId\": 80507,\n" +
                "        \"mainNumber\": 6,\n" +
                "        \"questionId\": \"5c45384def8fce45257834da\",\n" +
                "        \"order\": 23,\n" +
                "        \"studentAnswer\": null,\n" +
                "        \"answerType\": null,\n" +
                "        \"realAnswerType\": \"text\"\n" +
                "    }]\n" +
                "}";
    }
}
